"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const constants_1 = require("./constants");
class Client {
    constructor(apiKey) {
        this.apiKey = apiKey;
        this.httpClient = axios_1.default.create({
            baseURL: `${constants_1.API_URL_BASE}/${constants_1.API_VERSION}`,
            headers: {
                Authorization: `ApiKey ${this.apiKey}`,
            },
        });
    }
    createTenant(tenantId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.httpClient.post("/tenants", {
                    tenantId,
                });
                return response.data;
            }
            catch (e) {
                console.log("Error creating tenant in Warrant", e.response.data);
                throw e;
            }
        });
    }
    /**
     * Creates a user entity in Warrant with the specified `userId`.
     * Call this method whenever a new user is created in your application. After using this method to create a user entity
     * in Warrant, you can then use the isAuthorized method to authorize subsequent actions the specified user takes in your application.
     *
     * @param email
     * The user's email in your system.
     *
     * @param userId
     * The unique identifier to assign to this user in order to identify them in the future. This could be a unique identifier you
     * already use in your application (i.e. a database generated id). This parameter is optional and Warrant will generate a userId for
     * the user if it is not provided.
     */
    createUser(email, userId, tenantId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const requestBody = {
                    email,
                };
                if (userId) {
                    requestBody.userId = userId;
                }
                if (tenantId) {
                    requestBody.tenantId = tenantId;
                }
                const response = yield this.httpClient.post("/users", requestBody);
                return response.data;
            }
            catch (e) {
                console.log("Error creating user in Warrant", e.response.data);
                throw e;
            }
        });
    }
    createRole(roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.httpClient.post("/roles", {
                    roleId,
                });
                return response.data;
            }
            catch (e) {
                console.log("Error creating role in Warrant", e.response.data);
                throw e;
            }
        });
    }
    deleteRole(roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.httpClient.delete(`/roles/${roleId}`);
            }
            catch (e) {
                console.log("Error deleting role in Warrant", e.response.data);
                throw e;
            }
        });
    }
    createPermission(permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.httpClient.post("/permissions", {
                    permissionId,
                });
                return response.data;
            }
            catch (e) {
                console.log("Error creating permission in Warrant", e.response.data);
                throw e;
            }
        });
    }
    deletePermission(permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.httpClient.delete(`/permissions/${permissionId}`);
            }
            catch (e) {
                console.log("Error deleting permission in Warrant", e.response.data);
                throw e;
            }
        });
    }
    /**
     * Creates a warrant (authorization rule) in Warrant which specifies that the user (or userset) provided as `user` has the
     * given `relation` to the object of type `objectType` with id `objectId`.
     *
     * @param objectType The type of object this warrant is granting access to.
     * @param objectId The id of the object this warrant is granting access to.
     * @param relation The relation this warrant grants the user (or userset) on the object.
     * @param user The user (userId) or userset (objectType + objectId + relation) this warrant will apply to.
     * @returns The newly created warrant.
     */
    createWarrant(objectType, objectId, relation, user) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.httpClient.post("/warrants", {
                    objectType,
                    objectId,
                    relation,
                    user,
                });
                return response.data;
            }
            catch (e) {
                console.log("Error creating warrant in Warrant", e.response.data);
                throw e;
            }
        });
    }
    assignRoleToUser(userId, roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.httpClient.post(`/users/${userId}/roles/${roleId}`);
                return response.data;
            }
            catch (e) {
                console.log("Error assigning role to user in Warrant", e.response.data);
                throw e;
            }
        });
    }
    removeRoleFromUser(userId, roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.httpClient.delete(`/users/${userId}/roles/${roleId}`);
            }
            catch (e) {
                console.log("Error removing role from user in Warrant", e.response.data);
                throw e;
            }
        });
    }
    assignPermissionToUser(userId, permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.httpClient.post(`/users/${userId}/permissions/${permissionId}`);
                return response.data;
            }
            catch (e) {
                console.log("Error assigning permission to user in Warrant", e.response.data);
                throw e;
            }
        });
    }
    removePermissionFromUser(userId, permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.httpClient.delete(`/users/${userId}/permissions/${permissionId}`);
            }
            catch (e) {
                console.log("Error removing permission from user in Warrant", e.response.data);
                throw e;
            }
        });
    }
    /**
     * Creates a session in Warrant for the user with the specified `userId` and returns a session token which can be used to
     * make authorized requests to the Warrant API scoped to the specified user.
     *
     * @param userId The unique identifier assigned in Warrant to identify the specified user.
     * @returns The session token representing the specified user's active session. Use this session token (i.e in a Client application)
     * to make authorized requests to the Warrant API scoped only to the specified user.
     */
    createSession(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.httpClient.post(`/users/${userId}/sessions`);
                return response.data.token;
            }
            catch (e) {
                console.log("Error creating session for user in Warrant", e.response.data);
                throw e;
            }
        });
    }
    /**
     * Creates a self-service session in Warrant for the user with the specified userId and returns a session token which can be used to
     * make authorized requests to the Warrant API scoped to the specified tenantId.
     *
     * @param session A session object containing the userId, redirectUrl, and optional tenantId for which the self service session should be created.
     * @returns A url pointing to the self-service dashboard that will allow the specified user to make changes to the roles and permissions of users in their tenant.
     */
    createSelfServiceSession(session) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.httpClient.post(`/sessions`, Object.assign(Object.assign({}, session), { type: "ssdash" }));
                return response.data.url;
            }
            catch (e) {
                console.log("Error creating self-service session for user in Warrant", e.response.data);
                throw e;
            }
        });
    }
    /**
     * Checks against the warrants defined for your application to determine if the given `userId` has the given
     * `relation` to the given `objectId` of type `objectType`. Returns true if the the relation exists and false otherwise.
     * @param objectType The type of the object for which access is being checked.
     * @param objectId The id of the object for which access is being checked.
     * @param relation The relation to the object for which access is being checked.
     * @param userId The userId for which access is being checked.
     * @returns True if the `userId` has the given `relation` to `objectId` of type `objectType`, and false otherwise.
     */
    isAuthorized(objectType, objectId, relation, userId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.httpClient.post("/authorize", {
                    objectType,
                    objectId,
                    relation,
                    user: {
                        userId,
                    },
                });
                if (response.status === 200) {
                    return true;
                }
            }
            catch (e) {
                if (e.response.status !== 401) {
                    console.log("Error authorizing access through Warrant", e.response.data);
                }
                return false;
            }
        });
    }
    hasPermission(permissionId, userId) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.isAuthorized("permission", permissionId, "member", userId);
        });
    }
}
exports.default = Client;
